package com.diagbot.aggregate;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.facade.DeptConfigFacade;
import com.diagbot.facade.DiseaseConfigFacade;
import com.diagbot.facade.DrugConfigFacade;
import com.diagbot.facade.LisConfigFacade;
import com.diagbot.facade.OperationConfigFacade;
import com.diagbot.facade.PacsConfigFacade;
import com.diagbot.facade.TransfusionConfigFacade;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.Drug;
import com.diagbot.vo.ItemExt;
import com.diagbot.vo.SearchData;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @author: gaodm
 * @time: 2020/8/19 13:18
 */
@Component
public class AssembleAggregate {
    @Autowired
    private LisConfigFacade lisConfigFacade;
    @Autowired
    private PacsConfigFacade pacsConfigFacade;
    @Autowired
    private DiseaseConfigFacade diseaseConfigFacade;
    @Autowired
    private DrugConfigFacade drugConfigFacade;
    @Autowired
    private OperationConfigFacade operationConfigFacade;
    @Autowired
    private DeptConfigFacade deptConfigFacade;
    @Autowired
    private TransfusionConfigFacade transfusionConfigFacade;

    @DataProvider("assembleDataAll")
    public SearchData assembleDataAll(
            @InvokeParameter("searchData") SearchData searchData,
            @DataConsumer("lisConfigMap") Map<String, Map<String, Map<String, Long>>> lisConfigMap,
            @DataConsumer("pacsConfigMap") Map<String, Map<String, Long>> pacsConfigMap,
            @DataConsumer("diagConfigMap") Map<String, Map<String, Long>> diagConfigMap,
            @DataConsumer("drugConfigMap") Map<String, Map<String, Map<String, Long>>> drugConfigMap,
            @DataConsumer("operationConfigMap") Map<String, Map<String, Long>> operationConfigMap,
            @DataConsumer("deptConfigMap") Map<String, Map<String, Long>> deptConfigMap,
            @DataConsumer("transfusionConfigMap") Map<String, Map<String, Long>> transfusionConfigMap) {
        //检验
        if (MapUtils.isNotEmpty(lisConfigMap)) {
            if (ListUtil.isNotEmpty(searchData.getLis())) {
                List<Lis> convertLis = lisConvert(searchData.getLis(), lisConfigMap);
                searchData.setLis(convertLis);
            }
            if (ListUtil.isNotEmpty(searchData.getLisOrder())) {
                List<Lis> convertLis = lisConvert(searchData.getLisOrder(), lisConfigMap);
                searchData.setLisOrder(convertLis);
            }
        }
        //检查
        if (MapUtils.isNotEmpty(pacsConfigMap)) {
            if (ListUtil.isNotEmpty(searchData.getPacs())) {
                List<Pacs> convertPacs = pacsConvert(searchData.getPacs(), pacsConfigMap);
                searchData.setPacs(convertPacs);
            }
            if (ListUtil.isNotEmpty(searchData.getPacsOrder())) {
                List<Pacs> convertPacs = pacsConvert(searchData.getPacsOrder(), pacsConfigMap);
                searchData.setPacsOrder(convertPacs);
            }
        }
        //诊断
        if (MapUtils.isNotEmpty(diagConfigMap)) {
            if (ListUtil.isNotEmpty(searchData.getDiag())) {
                List<Item> convertItem = itemConvert(searchData.getDiag(), diagConfigMap);
                searchData.setDiag(convertItem);
            }
            if (ListUtil.isNotEmpty(searchData.getDiagOrder())) {
                List<Item> convertItem = itemConvert(searchData.getDiagOrder(), diagConfigMap);
                searchData.setDiagOrder(convertItem);
            }
            if (searchData.getDiseaseName() != null
                    && StringUtil.isNotBlank(searchData.getDiseaseName().getName())) {
                List<Item> convertItem = itemConvert(Lists.newArrayList(searchData.getDiseaseName()), diagConfigMap);
                searchData.setDiseaseName(convertItem.get(0));
            }
        }
        //药品
        if (MapUtils.isNotEmpty(drugConfigMap)) {
            if (ListUtil.isNotEmpty(searchData.getDrug())) {
                List<Drug> convertItem = drugConvert(searchData.getDrug(), drugConfigMap);
                searchData.setDrug(convertItem);
            }
            if (ListUtil.isNotEmpty(searchData.getDrugOrder())) {
                List<Drug> convertItem = drugConvert(searchData.getDrugOrder(), drugConfigMap);
                searchData.setDrugOrder(convertItem);
            }
        }
        //手术
        if (MapUtils.isNotEmpty(operationConfigMap)) {
            if (ListUtil.isNotEmpty(searchData.getOperation())) {
                List<Item> convertItem = itemConvert(searchData.getOperation(), operationConfigMap);
                searchData.setOperation(convertItem);
            }
            if (ListUtil.isNotEmpty(searchData.getOperationOrder())) {
                List<Item> convertItem = itemConvert(searchData.getOperationOrder(), operationConfigMap);
                searchData.setOperationOrder(convertItem);
            }
        }
        //科室
        if (MapUtils.isNotEmpty(deptConfigMap)) {
            if (ListUtil.isNotEmpty(searchData.getDept())) {
                List<Item> convertItem = itemConvert(searchData.getDept(), deptConfigMap);
                searchData.setDept(convertItem);
            }
        }
        //输血
        if (MapUtils.isNotEmpty(transfusionConfigMap)) {
            if (ListUtil.isNotEmpty(searchData.getTransfusion())) {
                List<ItemExt> convertItem = itemExtConvert(searchData.getTransfusion(), transfusionConfigMap);
                searchData.setTransfusion(convertItem);
            }
            if (ListUtil.isNotEmpty(searchData.getTransfusionOrder())) {
                List<ItemExt> convertItem = itemExtConvert(searchData.getTransfusionOrder(), transfusionConfigMap);
                searchData.setTransfusionOrder(convertItem);
            }
        }
        return searchData;
    }

    @DataProvider("lisConfigMap")
    public Map<String, Map<String, Map<String, Long>>> lisConfigMap(@InvokeParameter("searchData") SearchData searchData) {
        //化验公表转换
        List<String> lisHisNameList = Lists.newArrayList();
        //lis
        if (ListUtil.isNotEmpty(searchData.getLis())) {
            lisHisNameList.addAll(searchData.getLis()
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()) || StringUtil.isNotBlank(i.getUniqueName()))
                    .map(i -> i.getName())
                    .collect(Collectors.toList()));
        }
        //lisOrder
        if (ListUtil.isNotEmpty(searchData.getLisOrder())) {
            lisHisNameList.addAll(searchData.getLisOrder()
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()) || StringUtil.isNotBlank(i.getUniqueName()))
                    .map(i -> i.getName())
                    .collect(Collectors.toList()));
        }
        lisHisNameList = lisHisNameList
                .stream()
                .distinct()
                .collect(Collectors.toList());

        Map<String, Map<String, Map<String, Long>>> lisConfigMap = new HashMap<>();
        if (ListUtil.isNotEmpty(lisHisNameList)) {
            lisConfigMap
                    = lisConfigFacade.getConfigMap(searchData.getHospitalId(), lisHisNameList, null);
        }
        return lisConfigMap;
    }

    @DataProvider("pacsConfigMap")
    public Map<String, Map<String, Long>> pacsConfigMap(@InvokeParameter("searchData") SearchData searchData) {
        //辅检公表转换
        List<String> pacsHisNameList = Lists.newArrayList();
        //pacs
        if (ListUtil.isNotEmpty(searchData.getPacs())) {
            pacsHisNameList.addAll(searchData.getPacs()
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()) || StringUtil.isNotBlank(i.getUniqueName()))
                    .map(i -> i.getName())
                    .collect(Collectors.toList()));
        }
        //pacsOrder
        if (ListUtil.isNotEmpty(searchData.getPacsOrder())) {
            pacsHisNameList.addAll(searchData.getPacsOrder()
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()) || StringUtil.isNotBlank(i.getUniqueName()))
                    .map(i -> i.getName())
                    .collect(Collectors.toList()));
        }
        pacsHisNameList = pacsHisNameList
                .stream()
                .distinct()
                .collect(Collectors.toList());

        Map<String, Map<String, Long>> pacsConfigMap = new HashMap<>();
        if (ListUtil.isNotEmpty(pacsHisNameList)) {
            pacsConfigMap
                    = pacsConfigFacade.getConfigMap(searchData.getHospitalId(), pacsHisNameList, null);
        }
        return pacsConfigMap;
    }

    @DataProvider("diagConfigMap")
    public Map<String, Map<String, Long>> diagConfigMap(@InvokeParameter("searchData") SearchData searchData) {
        //诊断公表转换
        List<String> diagHisNameList = Lists.newArrayList();
        //diag
        if (ListUtil.isNotEmpty(searchData.getDiag())) {
            diagHisNameList.addAll(searchData.getDiag()
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()) || StringUtil.isNotBlank(i.getUniqueName()))
                    .map(i -> i.getName())
                    .collect(Collectors.toList()));
        }
        //diagOrder
        if (ListUtil.isNotEmpty(searchData.getDiagOrder())) {
            diagHisNameList.addAll(searchData.getDiagOrder()
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()) || StringUtil.isNotBlank(i.getUniqueName()))
                    .map(i -> i.getName())
                    .collect(Collectors.toList()));
        }
        //diseaseName
        if (searchData.getDiseaseName() != null
                && StringUtil.isNotBlank(searchData.getDiseaseName().getName())) {
            diagHisNameList.add(searchData.getDiseaseName().getName());
        }
        diagHisNameList = diagHisNameList
                .stream()
                .distinct()
                .collect(Collectors.toList());

        Map<String, Map<String, Long>> configMap = new HashMap<>();
        if (ListUtil.isNotEmpty(diagHisNameList)) {
            configMap
                    = diseaseConfigFacade.getConfigMap(searchData.getHospitalId(), diagHisNameList, null);
        }
        return configMap;
    }

    @DataProvider("drugConfigMap")
    public Map<String, Map<String, Map<String, Long>>> drugConfigMap(@InvokeParameter("searchData") SearchData searchData) {
        //药品公表转换
        List<String> drugHisNameList = Lists.newArrayList();
        //drug
        if (ListUtil.isNotEmpty(searchData.getDrug())) {
            drugHisNameList.addAll(searchData.getDrug()
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()) || StringUtil.isNotBlank(i.getUniqueName()))
                    .map(i -> i.getName())
                    .collect(Collectors.toList()));
        }
        //drugOrder
        if (ListUtil.isNotEmpty(searchData.getDrugOrder())) {
            drugHisNameList.addAll(searchData.getDrugOrder()
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()) || StringUtil.isNotBlank(i.getUniqueName()))
                    .map(i -> i.getName())
                    .collect(Collectors.toList()));
        }
        drugHisNameList = drugHisNameList
                .stream()
                .distinct()
                .collect(Collectors.toList());

        Map<String, Map<String, Map<String, Long>>> configMap = new HashMap<>();
        if (ListUtil.isNotEmpty(drugHisNameList)) {
            configMap
                    = drugConfigFacade.getConfigMap(searchData.getHospitalId(), drugHisNameList, null);
        }
        return configMap;
    }

    @DataProvider("operationConfigMap")
    public Map<String, Map<String, Long>> operationConfigMap(@InvokeParameter("searchData") SearchData searchData) {
        //手术公表转换
        List<String> operationHisNameList = Lists.newArrayList();
        //operation
        if (ListUtil.isNotEmpty(searchData.getOperation())) {
            operationHisNameList.addAll(searchData.getOperation()
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()) || StringUtil.isNotBlank(i.getUniqueName()))
                    .map(i -> i.getName())
                    .collect(Collectors.toList()));
        }
        //operationOrder
        if (ListUtil.isNotEmpty(searchData.getOperationOrder())) {
            operationHisNameList.addAll(searchData.getOperationOrder()
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()) || StringUtil.isNotBlank(i.getUniqueName()))
                    .map(i -> i.getName())
                    .collect(Collectors.toList()));
        }
        operationHisNameList = operationHisNameList
                .stream()
                .distinct()
                .collect(Collectors.toList());

        Map<String, Map<String, Long>> configMap = new HashMap<>();
        if (ListUtil.isNotEmpty(operationHisNameList)) {
            configMap
                    = operationConfigFacade.getConfigMap(searchData.getHospitalId(), operationHisNameList, null);
        }
        return configMap;
    }

    @DataProvider("deptConfigMap")
    public Map<String, Map<String, Long>> deptConfigMap(@InvokeParameter("searchData") SearchData searchData) {
        //科室公表转换
        List<String> deptHisNameList = Lists.newArrayList();
        //dept
        if (ListUtil.isNotEmpty(searchData.getDept())) {
            deptHisNameList.addAll(searchData.getDept()
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()) || StringUtil.isNotBlank(i.getUniqueName()))
                    .map(i -> i.getName())
                    .collect(Collectors.toList()));
        }
        deptHisNameList = deptHisNameList
                .stream()
                .distinct()
                .collect(Collectors.toList());

        Map<String, Map<String, Long>> configMap = new HashMap<>();

        if (ListUtil.isNotEmpty(deptHisNameList)) {
            configMap
                    = deptConfigFacade.getConfigMap(searchData.getHospitalId(), deptHisNameList, null);
        }
        return configMap;
    }

    @DataProvider("transfusionConfigMap")
    public Map<String, Map<String, Long>> transfusionConfigMap(@InvokeParameter("searchData") SearchData searchData) {
        //输血公表转换
        List<String> transfusionHisNameList = Lists.newArrayList();
        //transfusion
        if (ListUtil.isNotEmpty(searchData.getTransfusion())) {
            transfusionHisNameList.addAll(searchData.getTransfusion()
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()) || StringUtil.isNotBlank(i.getUniqueName()))
                    .map(i -> i.getName())
                    .collect(Collectors.toList()));
        }
        //transfusionOrder
        if (ListUtil.isNotEmpty(searchData.getTransfusionOrder())) {
            transfusionHisNameList.addAll(searchData.getTransfusionOrder()
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()) || StringUtil.isNotBlank(i.getUniqueName()))
                    .map(i -> i.getName())
                    .collect(Collectors.toList()));
        }
        transfusionHisNameList = transfusionHisNameList
                .stream()
                .distinct()
                .collect(Collectors.toList());

        Map<String, Map<String, Long>> configMap = new HashMap<>();

        if (ListUtil.isNotEmpty(transfusionHisNameList)) {
            configMap
                    = transfusionConfigFacade.getConfigMap(searchData.getHospitalId(), transfusionHisNameList, null);
        }
        return configMap;
    }

    /**
     * 化验公表转换
     *
     * @param lis
     * @param configMap
     * @return
     */
    public List<Lis> lisConvert(List<Lis> lis, Map<String, Map<String, Map<String, Long>>> configMap) {
        if (ListUtil.isEmpty(lis)) {
            return lis;
        }
        if (configMap == null || configMap.size() == 0) {
            return lis;
        }
        List<Lis> convertLis = Lists.newLinkedList();

        lis.forEach(lisRecord -> {
            if (StringUtil.isBlank(lisRecord.getName())
                    || StringUtil.isNotBlank(lisRecord.getUniqueName())) {
                convertLis.add(lisRecord);
            } else {
                Map<String, Long> uniqueNameMap = new HashMap<>();
                if (StringUtil.isBlank(lisRecord.getDetailName())) {
                    lisRecord.setDetailName("");
                }
                if (configMap.get(lisRecord.getName()) != null
                        && configMap.get(lisRecord.getName()).containsKey(lisRecord.getDetailName())) {
                    uniqueNameMap = configMap.get(lisRecord.getName()).get(lisRecord.getDetailName());
                }
                if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                    for (String uniqueName : uniqueNameMap.keySet()) {
                        Lis convertLisRecord = new Lis();
                        BeanUtil.copyProperties(lisRecord, convertLisRecord);
                        convertLisRecord.setUniqueName(uniqueName);
                        convertLis.add(convertLisRecord);
                    }
                }
            }
        });
        return convertLis;
    }

    /**
     * 辅检公表转换
     *
     * @param pacs
     * @param configMap
     * @return
     */
    public List<Pacs> pacsConvert(List<Pacs> pacs, Map<String, Map<String, Long>> configMap) {
        if (ListUtil.isEmpty(pacs)) {
            return pacs;
        }
        if (configMap == null || configMap.size() == 0) {
            return pacs;
        }
        List<Pacs> convertPacs = Lists.newLinkedList();

        pacs.forEach(pacsRecord -> {
            if (StringUtil.isBlank(pacsRecord.getName())
                    || StringUtil.isNotBlank(pacsRecord.getUniqueName())) {
                convertPacs.add(pacsRecord);
            } else {
                Map<String, Long> uniqueNameMap = new HashMap<>();
                if (configMap.get(pacsRecord.getName()) != null) {
                    uniqueNameMap = configMap.get(pacsRecord.getName());
                }
                if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                    for (String uniqueName : uniqueNameMap.keySet()) {
                        Pacs convertPacsRecord = new Pacs();
                        BeanUtil.copyProperties(pacsRecord, convertPacsRecord);
                        convertPacsRecord.setUniqueName(uniqueName);
                        convertPacs.add(convertPacsRecord);
                    }
                }
            }
        });
        return convertPacs;
    }

    /**
     * 药品公表转换
     *
     * @param drug
     * @param configMap
     * @return
     */
    public List<Drug> drugConvert(List<Drug> drug, Map<String, Map<String, Map<String, Long>>> configMap) {
        if (ListUtil.isEmpty(drug)) {
            return drug;
        }
        if (configMap == null || configMap.size() == 0) {
            return drug;
        }
        List<Drug> convertDrug = Lists.newLinkedList();

        drug.forEach(drugRecord -> {
            if (StringUtil.isBlank(drugRecord.getName())
                    || StringUtil.isNotBlank(drugRecord.getUniqueName())) {
                convertDrug.add(drugRecord);
            } else {
                Map<String, Long> uniqueNameMap = new HashMap<>();
                if (StringUtil.isBlank(drugRecord.getForm())) {
                    drugRecord.setForm("");
                }
                if (configMap.get(drugRecord.getName()) != null
                        && configMap.get(drugRecord.getName()).get(drugRecord.getForm()) != null) {
                    uniqueNameMap = configMap.get(drugRecord.getName()).get(drugRecord.getForm());
                }
                if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                    for (String uniqueName : uniqueNameMap.keySet()) {
                        Drug convertDrugRecord = new Drug();
                        BeanUtil.copyProperties(drugRecord, convertDrugRecord);
                        convertDrugRecord.setUniqueName(uniqueName);
                        convertDrug.add(convertDrugRecord);
                    }
                }
            }
        });
        return convertDrug;
    }

    /**
     * 手术、诊断、科室转换
     *
     * @param items
     * @param configMap
     * @return
     */
    public List<Item> itemConvert(List<Item> items, Map<String, Map<String, Long>> configMap) {
        if (ListUtil.isEmpty(items)) {
            return items;
        }
        if (configMap == null || configMap.size() == 0) {
            return items;
        }
        List<Item> convertItem = Lists.newLinkedList();
        items.forEach(item -> {
            if (StringUtil.isBlank(item.getName())
                    || StringUtil.isNotBlank(item.getUniqueName())) {
                convertItem.add(item);
            } else {
                Map<String, Long> uniqueNameMap = new HashMap<>();
                if (configMap.get(item.getName()) != null) {
                    uniqueNameMap = configMap.get(item.getName());
                }
                if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                    for (String uniqueName : uniqueNameMap.keySet()) {
                        Item convertItemRecord = new Item();
                        BeanUtil.copyProperties(item, convertItemRecord);
                        convertItemRecord.setUniqueName(uniqueName);
                        convertItem.add(convertItemRecord);
                    }
                }
            }
        });
        return convertItem;
    }

    /**
     * 输血转换
     *
     * @param items
     * @param configMap
     * @return
     */
    public List<ItemExt> itemExtConvert(List<ItemExt> items, Map<String, Map<String, Long>> configMap) {
        if (ListUtil.isEmpty(items)) {
            return items;
        }
        if (configMap == null || configMap.size() == 0) {
            return items;
        }
        List<ItemExt> convertItem = Lists.newLinkedList();
        items.forEach(item -> {
            if (StringUtil.isBlank(item.getName())
                    || StringUtil.isNotBlank(item.getUniqueName())) {
                convertItem.add(item);
            } else {
                Map<String, Long> uniqueNameMap = new HashMap<>();
                if (configMap.get(item.getName()) != null) {
                    uniqueNameMap = configMap.get(item.getName());
                }
                if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                    for (String uniqueName : uniqueNameMap.keySet()) {
                        ItemExt convertItemRecord = new ItemExt();
                        BeanUtil.copyProperties(item, convertItemRecord);
                        convertItemRecord.setUniqueName(uniqueName);
                        convertItem.add(convertItemRecord);
                    }
                }
            }
        });
        return convertItem;
    }
}