package com.diagbot.client.hystrix;

import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.PushPlanDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.vo.HasStaticKnowledgeVO;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.PushPlanVO;
import com.diagbot.vo.PushVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description: 病历质控客户端（请求失败熔断）
 * @author: gaodm
 * @time: 2020/7/28 9:29
 */
@Component
@Slf4j
public class CdssCoreHystrix implements CdssCoreClient {

    @Override
    public RespDTO<IndicationDTO> indication(@Valid @RequestBody IndicationPushVO indicationPushVO) {
        log.error("【hystrix】调用{}异常", "indication");
        return null;
    }

    @Override
    public RespDTO<PushDTO> push(@RequestBody @Valid PushVO pushVo) {
        log.error("【hystrix】调用{}异常", "push");
        return null;
    }

    /**
     * 推送持续检验检查计划
     */
    @Override
    public RespDTO<PushPlanDTO> pushPlan(@RequestBody PushPlanVO pushPlanVO){
        log.error("【hystrix】调用{}异常", "pushPlan");
        return null;
    }

    /**
     * 化验大项、化验小项、辅检、诊断、药品、手术等检索
     *
     * @param retrievalVO
     * @return
     */
    @Override
    public RespDTO<RetrievalDTO> index(@RequestBody @Valid RetrievalVO retrievalVO) {
        log.error("【hystrix】调用{}异常", "index");
        return null;
    }

    /**
     * 静态知识检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    @Override
    public RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndex(@RequestBody @Valid StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        log.error("【hystrix】调用{}异常", "staticKnowledgeIndex");
        return null;
    }

    /**
     * 更新是否有静态知识状态
     *
     * @param hasStaticKnowledgeVO
     * @return
     */
    @Override
    public RespDTO<Boolean> updateHasInfoStatus(@Valid @RequestBody HasStaticKnowledgeVO hasStaticKnowledgeVO) {
        log.error("【hystrix】调用{}异常", "updateHasInfoStatus");
        return null;
    }
}
