package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.dto.HospitalInfoDTO;
import com.diagbot.entity.HospitalInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.HospitalInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.HospitalCheckVO;
import com.diagbot.vo.HospitalInfoVO;
import com.diagbot.vo.HospitalSaveVO;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-07-29 11:08
 */
@Component
public class HospitalInfoFacade extends HospitalInfoServiceImpl {

    /**
     * @param hospitalInfoVO
     * @return
     */
    public HospitalInfoDTO getHospitalInfoById(HospitalInfoVO hospitalInfoVO) {
        QueryWrapper<HospitalInfo> queryWrapper = new QueryWrapper<HospitalInfo>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", hospitalInfoVO.getId());
        HospitalInfo datas = this.getOne(queryWrapper, false);
        HospitalInfoDTO data = new HospitalInfoDTO();
        BeanUtil.copyProperties(datas, data);
        return data;
    }

    public Boolean checkHospital(HospitalCheckVO hospitalCheckVO) {
        QueryWrapper<HospitalInfo> queryWrapper = new QueryWrapper<HospitalInfo>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("status", StatusEnum.Enable.getKey())
                .eq("id", hospitalCheckVO.getHospitalId());
        if (this.count(queryWrapper) > 0) {
            return true;
        }
        return false;
    }

    /**
     * 保存医院信息--医院端只能修改对接状态
     *
     * @param hospitalSaveVO
     * @return
     */
    public Boolean saveRecord(HospitalSaveVO hospitalSaveVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        if (!hospitalSaveVO.getId().equals(Long.valueOf(hospitalId))) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "不允许修改别家医院信息");
        }
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        UpdateWrapper<HospitalInfo> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("connect", hospitalSaveVO.getConnect())
                .set("modifier", userId)
                .set("gmt_modified", now)
                .eq("id", hospitalSaveVO.getId());
        return this.update(updateWrapper);
    }

    /**
     * 获取医院信息
     *
     * @return
     */
    public HospitalInfo getHospitalInfo() {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        HospitalInfo hospitalInfo = this.getById(Long.valueOf(hospitalId));
        return hospitalInfo;
    }
}