package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.PlanDetailDTO;
import com.diagbot.dto.PlanInfoDefaultDTO;
import com.diagbot.dto.PlanInfoPageDTO;
import com.diagbot.dto.SysPlanInfoDTO;
import com.diagbot.entity.Plan;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.PlanServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.HospitalPlanCancelVO;
import com.diagbot.vo.HospitalPlanDetailSaveVO;
import com.diagbot.vo.HospitalPlanIdVO;
import com.diagbot.vo.HospitalPlanPageVO;
import com.diagbot.vo.HospitalPlanSaveVO;
import com.diagbot.vo.HospitalPlanVO;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.PlanDetailCancelVO;
import com.diagbot.vo.PlanDetailSaveVO;
import com.diagbot.vo.PlanDetailSubSaveVO;
import com.diagbot.vo.PlanRevStopVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-07-27 19:23
 */
@Component
public class PlanFacade extends PlanServiceImpl {

    @Autowired
    PlanDetailFacade sysSetFacade;

    public List<SysPlanInfoDTO> getSysPlanInfoData(HospitalPlanVO hospitalPlanVO) {
        //查询该医院有没有相关方案配置
        List<SysPlanInfoDTO> sysPlanInfoDTO = new ArrayList<>();
        QueryWrapper<Plan> sysPlanInfo = new QueryWrapper<>();
        sysPlanInfo.eq("is_deleted", IsDeleteEnum.N.getKey());
        sysPlanInfo.eq("hospital_id", hospitalPlanVO.getHospitalId());
        sysPlanInfo.eq(StringUtil.isNotBlank(hospitalPlanVO.getPlanCode()), "plan_code", hospitalPlanVO.getPlanCode());
        sysPlanInfo.eq("plan_status", StatusEnum.Enable.getKey());
        List<Plan> sysPlanInfoData = list(sysPlanInfo);
        List<Long> planId = new ArrayList<>();
        Long hospitalId = hospitalPlanVO.getHospitalId();
        if (ListUtil.isNotEmpty(sysPlanInfoData)) {
            sysPlanInfoDTO = BeanUtil.listCopyTo(sysPlanInfoData, SysPlanInfoDTO.class);
            for (SysPlanInfoDTO data : sysPlanInfoDTO) {
                planId.add(data.getId());
            }
        }
        if(ListUtil.isEmpty(planId)){
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "该医院没有配置方案，请先配置方案！");
        }
        HospitalSetVO hospitalSetVO = new HospitalSetVO();
        hospitalSetVO.setHospitalId(hospitalId);
        hospitalSetVO.setPlanId(planId);
        List<PlanDetailDTO> sysSetInfoData = sysSetFacade.getSysSetInfoData(hospitalSetVO);
        //把父级和子级数据sysSetInfoData放在方案下
        Map<Long, List<PlanDetailDTO>> dataMap = sysSetInfoData.stream().collect(Collectors.groupingBy(PlanDetailDTO::getPlanId));
        for (Long key : dataMap.keySet()) {
            for (SysPlanInfoDTO sysPlanInfos : sysPlanInfoDTO) {
                if (sysPlanInfos.getId().equals(key)) {
                    sysPlanInfos.setSysSetInfo(dataMap.get(key));
                }
            }
        }

        return sysPlanInfoDTO;
    }

    /**
     * @param hospitalPlanSaveVO
     * @return
     */
    public boolean savePlanInfoDatas(HospitalPlanSaveVO hospitalPlanSaveVO) {
        Long planId = hospitalPlanSaveVO.getId();
        Date now = DateUtil.now();
        boolean res = false;
        // 校验名称是否相同
        int count = this.count(new QueryWrapper<Plan>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalPlanSaveVO.getHospitalId())
                .eq("plan_name", hospitalPlanSaveVO.getPlanName())
                .ne("id", hospitalPlanSaveVO.getId() == null ? -1 : hospitalPlanSaveVO.getId()));
        if (count > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "相同【医院】下，方案名称已存在");
        }
        int countTow = this.count(new QueryWrapper<Plan>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalPlanSaveVO.getHospitalId())
                .eq("plan_code", hospitalPlanSaveVO.getPlanCode())
                .ne("id", hospitalPlanSaveVO.getId() == null ? -1 : hospitalPlanSaveVO.getId()));
        if (countTow > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "相同【医院】下，方案编码已存在");
        }
        if (hospitalPlanSaveVO.getId() != null) {
            UpdateWrapper<Plan> planQuery = new UpdateWrapper<>();
            planQuery.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("id", hospitalPlanSaveVO.getId())
                    .set("gmt_modified", now)
                    .set("modifier", SysUserUtils.getCurrentPrincipleID())
                    .set("hospital_id", hospitalPlanSaveVO.getHospitalId())
                    .set("plan_name", hospitalPlanSaveVO.getPlanName())
                    .set("plan_code", hospitalPlanSaveVO.getPlanCode())
                    .set("plan_status", StatusEnum.Enable.getKey());
            res = update(new Plan(), planQuery);
        } else {
            Plan plan = new Plan();
            BeanUtil.copyProperties(hospitalPlanSaveVO, plan);
            plan.setGmtCreate(now);
            plan.setGmtModified(now);
            plan.setCreator(SysUserUtils.getCurrentPrincipleID());
            plan.setModifier(SysUserUtils.getCurrentPrincipleID());
            res = this.save(plan);
            planId = plan.getId();
        }
        PlanDetailSaveVO planDetailSaveVO = new PlanDetailSaveVO();
        planDetailSaveVO.setPlanId(planId);
        planDetailSaveVO.setHospitalId(hospitalPlanSaveVO.getHospitalId());
        List<HospitalPlanDetailSaveVO> planDetailParentNew = new ArrayList<>();
        List<HospitalPlanDetailSaveVO> planDetailParent = hospitalPlanSaveVO.getPlanDetailParent();
        if (ListUtil.isNotEmpty(planDetailParent)) {
            for (HospitalPlanDetailSaveVO data : planDetailParent) {
                HospitalPlanDetailSaveVO dataNew = new HospitalPlanDetailSaveVO();
                BeanUtil.copyProperties(data, dataNew);
                dataNew.setPlanId(planId);
                List<PlanDetailSubSaveVO> planDetailSubNew = new ArrayList<>();
                List<PlanDetailSubSaveVO> planDetailSub = data.getPlanDetailSub();
                if (ListUtil.isNotEmpty(planDetailSub)) {
                    for (PlanDetailSubSaveVO dataSub : planDetailSub) {
                        PlanDetailSubSaveVO dataSubNew = new PlanDetailSubSaveVO();
                        BeanUtil.copyProperties(dataSub, dataSubNew);
                        dataSubNew.setPlanId(planId);
                        planDetailSubNew.add(dataSubNew);
                    }
                    dataNew.setPlanDetailSub(planDetailSubNew);
                    planDetailParentNew.add(dataNew);
                }
            }
        }

        planDetailSaveVO.setPlanDetailParent(planDetailParentNew);
        res = sysSetFacade.savePlanDetail(planDetailSaveVO);
        return res;
    }

    public IPage<PlanInfoPageDTO> getPlanInfoPage(HospitalPlanPageVO hospitalPlanPageVO) {
        hospitalPlanPageVO.setHospitalId(Long.valueOf(SysUserUtils.getCurrentHospitalID()));
        IPage<PlanInfoPageDTO> data = getPlanInfoPageAll(hospitalPlanPageVO);
        List<PlanInfoPageDTO> planInfos = data.getRecords();
        // 取版本id查明细
        List<Long> ids = new ArrayList<>();
        if (planInfos != null) {
            // 当查出的数据不为空时，取到版本id，再去版本明细表中查询详细的信息
            for (PlanInfoPageDTO planInfo : planInfos) {
                ids.add(planInfo.getId());
            }
            // 获取明细信息
            List<PlanDetailDTO> PlanDetailDatas = sysSetFacade.getByPlanIds(ids);
            List<PlanDetailDTO> sysSetInfoData = BeanUtil.listCopyTo(PlanDetailDatas, PlanDetailDTO.class);
            List<PlanDetailDTO> planDetailParent = new ArrayList<>();//父级数据
            List<PlanDetailDTO> planDetailSub = new ArrayList<>();//子级数据
            //分离数据，-1分父级菜单
            for (PlanDetailDTO dataDeta : sysSetInfoData) {
                if (dataDeta.getParentId() == (-1)) {
                    planDetailParent.add(dataDeta);
                } else {
                    planDetailSub.add(dataDeta);
                }
            }
            //把子级数据，放在父级下
            Map<Long, List<PlanDetailDTO>> mapParentId = planDetailSub.stream().collect(Collectors.groupingBy(PlanDetailDTO::getParentId));
            for (Long key : mapParentId.keySet()) {
                for (PlanDetailDTO sysPlanInfos : planDetailParent) {
                    if (sysPlanInfos.getId().equals(key)) {
                        sysPlanInfos.setPlanDetails(mapParentId.get(key));
                    }
                }
            }

            Map<Long, List<PlanDetailDTO>> map = new HashMap<>();
            // 获取所有用户开通的产品信息
            map = EntityUtil.makeEntityListMap(planDetailParent, "planId");
            if (map.size() > 0) {
                for (PlanInfoPageDTO planInfo : planInfos) {
                    List<PlanDetailDTO> planDetails = map.get(planInfo.getId());
                    if (ListUtil.isNotEmpty(planDetails)) {
                        planInfo.setSysSetInfo(planDetails);
                    }
                }
            }
        }
        return data.setRecords(planInfos);
    }

    public boolean cancelPlanData(HospitalPlanCancelVO hospitalPlanCancelVO) {
        PlanDetailCancelVO planDetailCancelVO = new PlanDetailCancelVO();
        planDetailCancelVO.setHospitalId(hospitalPlanCancelVO.getHospitalId());
        planDetailCancelVO.setPlanId(hospitalPlanCancelVO.getPlanId());
        boolean res = false;
        // 1.先判断数据是否存在有效
        Plan plan = checkPlan(hospitalPlanCancelVO.getPlanId());
        if (plan.getPlanStatus() == 1) {
            throw new CommonException(CommonErrorCode.FAIL, "该方案启用中，不能删除！");
        }
        UpdateWrapper<Plan> planNew = new UpdateWrapper<>();
        planNew
                .eq("id", hospitalPlanCancelVO.getPlanId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalPlanCancelVO.getHospitalId())
                .set("is_deleted", IsDeleteEnum.Y.getKey())
                .set("modifier", UserUtils.getCurrentPrincipleID())
                .set("gmt_modified", DateUtil.now());
        res = update(new Plan(), planNew);
        //再删除明细
        sysSetFacade.cancelPlanDetail(planDetailCancelVO);

        return res;
    }

    public boolean revStopPlan(PlanRevStopVO planRevStopVO) {
        checkPlan(planRevStopVO.getId());
        UpdateWrapper<Plan> planNew = new UpdateWrapper<>();
        planNew
                .eq("id", planRevStopVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .set("plan_status", planRevStopVO.getStatus())
                .set("modifier", UserUtils.getCurrentPrincipleID())
                .set("gmt_modified", DateUtil.now());
        return update(new Plan(), planNew);
    }

    /**
     * 判断数据是否存在有效
     *
     * @param id
     */
    private Plan checkPlan(Long id) {
        // 1.先判断数据是否存在有效
        QueryWrapper<Plan> PlanFand = new QueryWrapper<>();
        PlanFand.eq("is_deleted", IsDeleteEnum.N.getKey()).eq("id", id);
        Plan plan = getOne(PlanFand, false);
        if (null == plan) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "该数据不存在");
        }
        return plan;
    }


    public PlanInfoDefaultDTO getDefaultPlan() {
        PlanInfoDefaultDTO data = new PlanInfoDefaultDTO();
        QueryWrapper<Plan> planFand = new QueryWrapper<>();
        planFand.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("plan_code", "plan_default")
                .eq("hospital_id", -1L);
        Plan plan = getOne(planFand, false);
        BeanUtil.copyProperties(plan, data);
        List<Long> planId = new ArrayList<>();
        planId.add(plan.getId());
        HospitalSetVO hospitalSetVO = new HospitalSetVO();
        hospitalSetVO.setHospitalId(-1L);
        hospitalSetVO.setPlanId(planId);
        List<PlanDetailDTO> sysSetInfoData = sysSetFacade.getSysSetInfoData(hospitalSetVO);
        data.setPlanDetailDefault(sysSetInfoData);
        return data;
    }

    public List<SysPlanInfoDTO> getPlanInfoIds(HospitalPlanIdVO hospitalPlanIdVO) {
        //查询该医院有没有相关方案配置
        List<SysPlanInfoDTO> sysPlanInfoDTO = new ArrayList<>();
        QueryWrapper<Plan> sysPlanInfo = new QueryWrapper<>();
        sysPlanInfo.eq("is_deleted", IsDeleteEnum.N.getKey());
        sysPlanInfo.eq("hospital_id", hospitalPlanIdVO.getHospitalId());
        sysPlanInfo.eq("id", hospitalPlanIdVO.getId());
        List<Plan> sysPlanInfoData = list(sysPlanInfo);
        List<Long> planId = new ArrayList<>();
        Long hospitalId = hospitalPlanIdVO.getHospitalId();
        if (ListUtil.isNotEmpty(sysPlanInfoData)) {
            sysPlanInfoDTO = BeanUtil.listCopyTo(sysPlanInfoData, SysPlanInfoDTO.class);
            for (SysPlanInfoDTO data : sysPlanInfoDTO) {
                planId.add(data.getId());
            }
        }
        if(ListUtil.isEmpty(planId)){
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "该数据不存在！");
        }
        HospitalSetVO hospitalSetVO = new HospitalSetVO();
        hospitalSetVO.setHospitalId(hospitalId);
        hospitalSetVO.setPlanId(planId);
        List<PlanDetailDTO> sysSetInfoData = sysSetFacade.getSysSetInfoData(hospitalSetVO);
        //把父级和子级数据sysSetInfoData放在方案下
        Map<Long, List<PlanDetailDTO>> dataMap = sysSetInfoData.stream().collect(Collectors.groupingBy(PlanDetailDTO::getPlanId));
        for (Long key : dataMap.keySet()) {
            for (SysPlanInfoDTO sysPlanInfos : sysPlanInfoDTO) {
                if (sysPlanInfos.getId().equals(key)) {
                    sysPlanInfos.setSysSetInfo(dataMap.get(key));
                }
            }
        }

        return sysPlanInfoDTO;
    }
}
