package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.DiseaseConfig;
import com.diagbot.facade.DiseaseConfigFacade;
import com.diagbot.vo.DiseaseConfigListVO;
import com.diagbot.vo.DiseaseConfigPageVO;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

/**
 * <p>
 * 诊断映射表 前端控制器
 * </p>
 *
 * @author zhaops
 * @since 2020-07-28
 */
@RestController
@RequestMapping("/tran/diseaseConfig")
@Api(value = "诊断公表映射API", tags = { "诊断公表映射API" })
@SuppressWarnings("unchecked")
public class DiseaseConfigController {
    @Autowired
    private DiseaseConfigFacade diseaseConfigFacade;

    /**
     * 映射关系是否已存在
     *
     * @param diseaseConfig
     * @return
     */
    @ApiOperation(value = "映射关系是否已存在[by:zhaops]", notes = "")
    @PostMapping("/isExistRecord")
    @SysLogger("isExistRecord")
    public RespDTO<Boolean> isExistRecord(@RequestBody @Valid DiseaseConfig diseaseConfig) {
        Boolean data = diseaseConfigFacade.isExistRecord(diseaseConfig);
        return RespDTO.onSuc(data);
    }

    /**
     * 保存或修改映射关系
     *
     * @param diseaseConfig
     * @return
     */
    @ApiOperation(value = "保存或修改映射关系[by:zhaops]", notes = "")
    @PostMapping("/saveOrUpdateRecord")
    @SysLogger("saveOrUpdateRecord")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecord(@RequestBody @Valid DiseaseConfig diseaseConfig) {
        Boolean data = diseaseConfigFacade.saveOrUpdateRecord(diseaseConfig);
        return RespDTO.onSuc(data);
    }

    /**
     * 批量保存或修改映射关系
     *
     * @param diseaseConfigListVO
     * @return
     */
    @ApiOperation(value = "批量保存或修改映射关系[by:zhaops]", notes = "")
    @PostMapping("/saveOrUpdateRecords")
    @SysLogger("saveOrUpdateRecords")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecords(@RequestBody @Valid DiseaseConfigListVO diseaseConfigListVO) {
        Boolean data = diseaseConfigFacade.saveOrUpdateRecords(diseaseConfigListVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 删除映射关系
     *
     * @param idVO
     * @return
     */
    @ApiOperation(value = "删除映射关系[by:zhaops]", notes = "")
    @PostMapping("/deleteRecord")
    @SysLogger("deleteRecord")
    @Transactional
    public RespDTO<Boolean> deleteRecord(@RequestBody @Valid IdVO idVO) {
        Boolean data = diseaseConfigFacade.deleteRecord(idVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 批量删除映射关系
     *
     * @param idListVO
     * @return
     */
    @ApiOperation(value = "批量删除映射关系[by:zhaops]", notes = "")
    @PostMapping("/deleteRecords")
    @SysLogger("deleteRecords")
    @Transactional
    public RespDTO<Boolean> deleteRecords(@RequestBody @Valid IdListVO idListVO) {
        Boolean data = diseaseConfigFacade.deleteRecords(idListVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 分页查询
     *
     * @param diseaseConfigPageVO
     * @return
     */
    @ApiOperation(value = "分页查询[by:zhaops]", notes = "")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    public RespDTO<DiseaseConfig> getPage(@RequestBody @Valid DiseaseConfigPageVO diseaseConfigPageVO) {
        IPage<DiseaseConfig> data = diseaseConfigFacade.getPage(diseaseConfigPageVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 公表数据导入
     *
     * @param file
     * @return
     */
    @ApiOperation(value = "公表数据导入[by:zhaops]",
            notes = "")
    @PostMapping("/importExcel")
    @SysLogger("importExcel")
    @Transactional
    public void importExcel(@RequestParam("file") MultipartFile file) {
        diseaseConfigFacade.importExcel(file);
    }

    /**
     * 数据导出
     *
     * @return
     */
    @ApiOperation(value = "数据导出[by:gaodm]",
            notes = "")
    @PostMapping("/exportExcel")
    @SysLogger("exportExcel")
    public void exportExcel(HttpServletResponse response) {
        diseaseConfigFacade.exportExcel(response);
    }
}
