package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.TransfusionConfig;
import com.diagbot.facade.TransfusionConfigFacade;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.TransfusionConfigListVO;
import com.diagbot.vo.TransfusionConfigPageVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

/**
 * <p>
 * 输血映射表 前端控制器
 * </p>
 *
 * @author zhaops
 * @since 2020-08-31
 */
@RestController
@RequestMapping("/tran/transfusionConfig")
@Api(value = "输血公表映射API", tags = { "输血公表映射API" })
@SuppressWarnings("unchecked")
public class TransfusionConfigController {
    @Autowired
    private TransfusionConfigFacade transfusionConfigFacade;

    /**
     * 映射关系是否已存在
     *
     * @param transfusionConfig
     * @return
     */
    @ApiOperation(value = "映射关系是否已存在[by:zhaops]", notes = "")
    @PostMapping("/isExistRecord")
    @SysLogger("isExistRecord")
    public RespDTO<Boolean> isExistRecord(@RequestBody @Valid TransfusionConfig transfusionConfig) {
        Boolean data = transfusionConfigFacade.isExistRecord(transfusionConfig);
        return RespDTO.onSuc(data);
    }

    /**
     * 保存或修改映射关系
     *
     * @param transfusionConfig
     * @return
     */
    @ApiOperation(value = "保存或修改映射关系[by:zhaops]", notes = "")
    @PostMapping("/saveOrUpdateRecord")
    @SysLogger("saveOrUpdateRecord")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecord(@RequestBody @Valid TransfusionConfig transfusionConfig) {
        Boolean data = transfusionConfigFacade.saveOrUpdateRecord(transfusionConfig);
        return RespDTO.onSuc(data);
    }

    /**
     * 批量保存或修改映射关系
     *
     * @param transfusionConfigListVO
     * @return
     */
    @ApiOperation(value = "批量保存或修改映射关系[by:zhaops]", notes = "")
    @PostMapping("/saveOrUpdateRecords")
    @SysLogger("saveOrUpdateRecords")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecords(@RequestBody @Valid TransfusionConfigListVO transfusionConfigListVO) {
        Boolean data = transfusionConfigFacade.saveOrUpdateRecords(transfusionConfigListVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 删除映射关系
     *
     * @param idVO
     * @return
     */
    @ApiOperation(value = "删除映射关系[by:zhaops]", notes = "")
    @PostMapping("/deleteRecord")
    @SysLogger("deleteRecord")
    @Transactional
    public RespDTO<Boolean> deleteRecord(@RequestBody @Valid IdVO idVO) {
        Boolean data = transfusionConfigFacade.deleteRecord(idVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 批量删除映射关系
     *
     * @param idListVO
     * @return
     */
    @ApiOperation(value = "批量删除映射关系[by:zhaops]", notes = "")
    @PostMapping("/deleteRecords")
    @SysLogger("deleteRecords")
    @Transactional
    public RespDTO<Boolean> deleteRecords(@RequestBody @Valid IdListVO idListVO) {
        Boolean data = transfusionConfigFacade.deleteRecords(idListVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 分页查询
     *
     * @param transfusionConfigPageVO
     * @return
     */
    @ApiOperation(value = "分页查询[by:zhaops]", notes = "")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    public RespDTO<TransfusionConfig> getPage(@RequestBody @Valid TransfusionConfigPageVO transfusionConfigPageVO) {
        IPage<TransfusionConfig> data = transfusionConfigFacade.getPage(transfusionConfigPageVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 公表数据导入
     *
     * @param file
     * @return
     */
    @ApiOperation(value = "公表数据导入[by:zhaops]",
            notes = "")
    @PostMapping("/importExcel")
    @SysLogger("importExcel")
    @Transactional
    public void importExcel(@RequestParam("file") MultipartFile file) {
        transfusionConfigFacade.importExcel(file);
    }

    /**
     * 数据导出
     *
     * @return
     */
    @ApiOperation(value = "数据导出[by:gaodm]",
            notes = "")
    @PostMapping("/exportExcel")
    @SysLogger("exportExcel")
    public void exportExcel(HttpServletResponse response) {
        transfusionConfigFacade.exportExcel(response);
    }
}